import io
import json
import logging
from fdk import response
import oci
import oci_dns_instance_update


def handler(ctx, data: io.BytesIO=None):
    try:
        
        try: 
            app_context_variables = dict(ctx.Config())
            private_zoned_ocid_list = app_context_variables['TARGET_ZONES_OCID']
        except Exception as ex:
            logging.ERROR('Getting ctx config - '+str(ex))
            raise


        # get the info from the event service
        body = json.loads(data.getvalue())
        logging.getLogger().info("Body is: " + str(body))
        
        # get compartment id and instance name
        compartment_id = body["data"]["compartmentId"]
        #resource_name = body["data"]["resourceName"]
        resource_id = body["data"]["resourceId"]
                      
        # see if instance was created or terminated
        action_type = body["eventType"]
        logging.getLogger().info("action_type is: " + str(action_type))

        # setup authentication as resource Principal
        signer = oci.auth.signers.get_resource_principals_signer()
        compute_client = oci.core.ComputeClient(config={}, signer=signer)
        network_client = oci.core.VirtualNetworkClient(config={}, signer=signer)


        host_list = oci_dns_instance_update.getInstanceData(compute_client,network_client,resource_id,compartment_id )
        logging.getLogger().info("Selected Zone List: " + private_zoned_ocid_list)
        ocid_zone_list = oci_dns_instance_update.get_dns_zones_and_regions(private_zoned_ocid_list)
        for host in host_list:
            logging.getLogger().info(f"Private Zone Record: HOSTNAME: {host['HOSTNAME']}, IP: {host['IP']}")
            hostname = host['HOSTNAME']
            ipaddress = host['IP']
            for zoneocid in ocid_zone_list:
                logging.getLogger().info("Working on record on Zone OCID : "+zoneocid['OCID']+" - On region: "+zoneocid['REGION'])
                config_regions = {
                    "region": zoneocid['REGION']
                    }
                dns_client = oci.dns.DnsClient(config=config_regions, signer=signer)
                domainPz = dns_client.get_zone(zone_name_or_id=zoneocid['OCID']).data.name
                record_name = hostname+"."+domainPz
                logging.getLogger().info("About to update the DNS Record : "+record_name)
                response_call = oci_dns_instance_update.update_a_record(dns_client, zoneocid['OCID'],record_name , ipaddress )
                #logging.getLogger().info(response_call.data)
                logging.getLogger().info("Response is: " + str(response_call.data))

                
    except (Exception, ValueError) as ex:
        logging.getLogger().error("There was an error: " + str(ex))
    return response.Response(
        ctx, response_data=json.dumps(
            {"message": "The function executed successfully"}),
        headers={"Content-Type": "application/json"}
    )
  
  